@@ -26,7 +26,6 @@ extern crate rayon;
 mod rayon;
 
 use std::{
-    borrow::Cow,
     fs::{File, Metadata},
     io::{stdin, stdout, BufWriter, Read, Write},
     path::Path,
@@ -155,23 +154,27 @@ impl RawImage {
 
     /// Create an optimized png from the raw image data using the options provided
     pub fn create_optimized_png(&self, opts: &Options) -> PngResult<Vec<u8>> {
+        let mut opts = opts.to_owned();
+        let mut aux_chunks: Vec<_> = self
+            .aux_chunks
+            .iter()
+            .filter(|c| opts.strip.keep(&c.name))
+            .cloned()
+            .collect();
+        preprocess_chunks(&mut aux_chunks, &mut opts);
+
         let deadline = Arc::new(Deadline::new(opts.timeout));
-        let Some(result) = optimize_raw(self.png.clone(), opts, deadline, None) else {
+        let Some(result) = optimize_raw(self.png.clone(), &opts, deadline, None) else {
             return Err(PngError::new("Failed to optimize input data"));
         };
 
         let mut png = PngData {
             raw: result.image,
-            idat_data: result.idat_data,
-            aux_chunks: self
-                .aux_chunks
-                .iter()
-                .filter(|c| opts.strip.keep(&c.name))
-                .cloned()
-                .collect(),
+            idat_data: result.data,
+            aux_chunks,
             frames: Vec::new(),
         };
-        postprocess_chunks(&mut png, opts, &self.png.ihdr);
+        postprocess_chunks(&mut png.aux_chunks, &png.raw.ihdr, &self.png.ihdr);
 
         Ok(png.output())
     }
@@ -180,7 +183,7 @@ impl RawImage {
 /// Perform optimization on the input file using the options provided
 pub fn optimize(input: &InFile, output: &OutFile, opts: &Options) -> PngResult<()> {
     // Read in the file and try to decode as PNG.
-    info!("Processing: {}", input);
+    info!("Processing: {input}");
 
     let deadline = Arc::new(Deadline::new(opts.timeout));
 
@@ -204,7 +207,7 @@ pub fn optimize(input: &InFile, output: &OutFile, opts: &Options) -> PngResult<(
                         ))
                     })
                     .map(Some)?;
-                trace!("preserving metadata: {:?}", opt_metadata_preserved);
+                trace!("preserving metadata: {opt_metadata_preserved:?}");
             } else {
                 opt_metadata_preserved = None;
             }
@@ -233,7 +236,7 @@ pub fn optimize(input: &InFile, output: &OutFile, opts: &Options) -> PngResult<(
             (OutFile::Path { path, .. }, InFile::Path(ref input_path))
                 if path.as_ref().map_or(true, |p| p == input_path) =>
             {
-                info!("{}: Could not optimize further, no change written", input);
+                info!("{input}: Could not optimize further, no change written");
                 return Ok(());
             }
             _ => {
@@ -258,7 +261,7 @@ pub fn optimize(input: &InFile, output: &OutFile, opts: &Options) -> PngResult<(
 
     match (output, input) {
         (OutFile::None, _) => {
-            info!("{}: Running in pretend mode, no output", savings);
+            info!("{savings}: Running in pretend mode, no output");
         }
         (&OutFile::StdOut, _) | (&OutFile::Path { path: None, .. }, &InFile::StdIn) => {
             let mut buffer = BufWriter::new(stdout());
@@ -343,35 +346,24 @@ fn optimize_png(
         raw.ihdr.width, raw.ihdr.height
     );
     report_format("    ", &raw);
-    debug!("    IDAT size = {} bytes", idat_original_size);
-    debug!("    File size = {} bytes", file_original_size);
+    debug!("    IDAT size = {idat_original_size} bytes");
+    debug!("    File size = {file_original_size} bytes");
+
+    let mut opts = opts.to_owned();
+    preprocess_chunks(&mut png.aux_chunks, &mut opts);
 
-    // Check for APNG by presence of acTL chunk
-    let opts = if png.aux_chunks.iter().any(|c| &c.name == b"acTL") {
-        warn!("APNG detected, disabling all reductions");
-        let mut opts = opts.to_owned();
-        opts.interlace = None;
-        opts.bit_depth_reduction = false;
-        opts.color_type_reduction = false;
-        opts.palette_reduction = false;
-        opts.grayscale_reduction = false;
-        Cow::Owned(opts)
-    } else {
-        Cow::Borrowed(opts)
-    };
     let max_size = if opts.force {
         None
     } else {
-        Some(png.estimated_output_size())
+        Some(png.raw.estimated_output_size(&png.idat_data))
     };
     if let Some(result) = optimize_raw(raw.clone(), &opts, deadline.clone(), max_size) {
         png.raw = result.image;
-        png.idat_data = result.idat_data;
+        png.idat_data = result.data;
         recompress_frames(png, &opts, deadline, result.filter)?;
+        postprocess_chunks(&mut png.aux_chunks, &png.raw.ihdr, &raw.ihdr);
     }
 
-    postprocess_chunks(png, &opts, &raw.ihdr);
-
     let output = png.output();
 
     if idat_original_size >= png.idat_data.len() {
@@ -441,7 +433,13 @@ fn optimize_raw(
         indexset! {RowFilter::None, RowFilter::Bigrams}
     };
     // This will collect all versions of images and pick one that compresses best
-    let eval = Evaluator::new(deadline.clone(), eval_filters.clone(), eval_deflater, false);
+    let eval = Evaluator::new(
+        deadline.clone(),
+        eval_filters.clone(),
+        eval_deflater,
+        false,
+        opts.deflate == eval_deflater,
+    );
     let mut new_image = perform_reductions(image.clone(), opts, &deadline, &eval);
     let eval_result = eval.get_best_candidate();
     if let Some(ref result) = eval_result {
@@ -472,7 +470,9 @@ fn optimize_raw(
         (eval_result?, eval_deflater)
     };
 
-    if max_size.map_or(true, |max_size| result.estimated_output_size() < max_size) {
+    if result.data_is_compressed
+        && max_size.map_or(true, |max_size| result.estimated_output_size < max_size)
+    {
         debug!("Found better result:");
         debug!("    {}, f = {}", deflater, result.filter);
         return Some(result);
@@ -507,35 +507,36 @@ fn perform_trials(
                 filters,
                 eval_deflater,
                 opts.optimize_alpha,
+                opts.deflate == eval_deflater,
             );
             if let Some(result) = &eval_result {
-                eval.set_best_size(result.estimated_output_size());
+                eval.set_best_size(result.estimated_output_size);
             }
             eval.try_image(image.clone());
             if let Some(result) = eval.get_best_candidate() {
                 eval_result = Some(result);
             }
         }
-        if opts.deflate == eval_deflater {
-            // No further compression required
-            return eval_result;
-        }
 
         // We should have a result here - fail if not (e.g. deadline passed)
         let mut result = eval_result?;
 
-        // Recompress with the main deflater
-        debug!("Trying filter {} with {}", result.filter, opts.deflate);
-        match opts.deflate.deflate(&result.filtered, max_size) {
-            Ok(idat_data) => {
-                result.idat_data = idat_data;
-                trace!("{} bytes", result.estimated_output_size());
-            }
-            Err(PngError::DeflatedDataTooLong(bytes)) => {
-                trace!(">{bytes} bytes");
-            }
-            Err(_) => (),
-        };
+        if !result.data_is_compressed {
+            // Compress with the main deflater
+            debug!("Trying filter {} with {}", result.filter, opts.deflate);
+            match opts.deflate.deflate(&result.data, max_size) {
+                Ok(idat_data) => {
+                    result.estimated_output_size = result.image.estimated_output_size(&idat_data);
+                    result.data = idat_data;
+                    result.data_is_compressed = true;
+                    trace!("{} bytes", result.estimated_output_size);
+                }
+                Err(PngError::DeflatedDataTooLong(bytes)) => {
+                    trace!(">{bytes} bytes");
+                }
+                Err(_) => (),
+            };
+        }
         return Some(result);
     }
 
@@ -553,7 +554,7 @@ fn perform_trials(
     }
 
     debug!("Trying {} filters with {}", filters.len(), opts.deflate);
-    let eval = Evaluator::new(deadline, filters, opts.deflate, opts.optimize_alpha);
+    let eval = Evaluator::new(deadline, filters, opts.deflate, opts.optimize_alpha, true);
     if let Some(max_size) = max_size {
         eval.set_best_size(max_size);
     }
@@ -619,61 +620,6 @@ fn report_format(prefix: &str, png: &PngImage) {
     );
 }
 
-/// Perform cleanup of certain chunks from the `PngData` object, after optimization has been completed
-fn postprocess_chunks(png: &mut PngData, opts: &Options, orig_ihdr: &IhdrData) {
-    if let Some(iccp_idx) = png.aux_chunks.iter().position(|c| &c.name == b"iCCP") {
-        // See if we can replace an iCCP chunk with an sRGB chunk
-        let may_replace_iccp = opts.strip != StripChunks::None && opts.strip.keep(b"sRGB");
-        if may_replace_iccp && png.aux_chunks.iter().any(|c| &c.name == b"sRGB") {
-            // Files aren't supposed to have both chunks, so we chose to honor sRGB
-            trace!("Removing iCCP chunk due to conflict with sRGB chunk");
-            png.aux_chunks.remove(iccp_idx);
-        } else if let Some(icc) = extract_icc(&png.aux_chunks[iccp_idx]) {
-            let intent = if may_replace_iccp {
-                srgb_rendering_intent(&icc)
-            } else {
-                None
-            };
-            // sRGB-like profile can be replaced with an sRGB chunk with the same rendering intent
-            if let Some(intent) = intent {
-                trace!("Replacing iCCP chunk with equivalent sRGB chunk");
-                png.aux_chunks[iccp_idx] = Chunk {
-                    name: *b"sRGB",
-                    data: vec![intent],
-                };
-            } else if opts.idat_recoding {
-                // Try recompressing the profile
-                let cur_len = png.aux_chunks[iccp_idx].data.len();
-                if let Ok(iccp) = make_iccp(&icc, opts.deflate, Some(cur_len - 1)) {
-                    debug!(
-                        "Recompressed iCCP chunk: {} ({} bytes decrease)",
-                        iccp.data.len(),
-                        cur_len - iccp.data.len()
-                    );
-                    png.aux_chunks[iccp_idx] = iccp;
-                }
-            }
-        }
-    }
-
-    // If the depth/color type has changed, some chunks may be invalid and should be dropped
-    // While these could potentially be converted, they have no known use case today and are
-    // generally more trouble than they're worth
-    let ihdr = &png.raw.ihdr;
-    if orig_ihdr.bit_depth != ihdr.bit_depth || orig_ihdr.color_type != ihdr.color_type {
-        png.aux_chunks.retain(|c| {
-            let invalid = &c.name == b"bKGD" || &c.name == b"sBIT" || &c.name == b"hIST";
-            if invalid {
-                warn!(
-                    "Removing {} chunk as it no longer matches the image data",
-                    std::str::from_utf8(&c.name).unwrap()
-                );
-            }
-            !invalid
-        });
-    }
-}
-
 /// Recompress the additional frames of an APNG
 fn recompress_frames(
     png: &mut PngData,
@@ -735,11 +681,7 @@ fn copy_times(_: &Metadata, _: &Path) -> PngResult<()> {
 fn copy_times(input_path_meta: &Metadata, out_path: &Path) -> PngResult<()> {
     let atime = filetime::FileTime::from_last_access_time(input_path_meta);
     let mtime = filetime::FileTime::from_last_modification_time(input_path_meta);
-    trace!(
-        "attempting to set file times: atime: {:?}, mtime: {:?}",
-        atime,
-        mtime
-    );
+    trace!("attempting to set file times: atime: {atime:?}, mtime: {mtime:?}");
     filetime::set_file_times(out_path, atime, mtime).map_err(|err_io| {
         PngError::new(&format!(
             "unable to set file times on {out_path:?}: {err_io}"
