@@ -386,6 +386,18 @@ fn rgb_8_should_be_palette_8() {
     );
 }
 
+#[test]
+fn rgb_trns_8_should_be_palette_8() {
+    test_it_converts(
+        "tests/files/rgb_trns_8_should_be_palette_8.png",
+        false,
+        RGB,
+        BitDepth::Eight,
+        INDEXED,
+        BitDepth::Eight,
+    );
+}
+
 #[test]
 fn rgb_16_should_be_palette_4() {
     test_it_converts(
@@ -566,6 +578,18 @@ fn palette_8_should_be_grayscale_8() {
     );
 }
 
+#[test]
+fn palette_2_should_be_grayscale_alpha_8() {
+    test_it_converts(
+        "tests/files/palette_2_should_be_grayscale_alpha_8.png",
+        true,
+        INDEXED,
+        BitDepth::Two,
+        GRAYSCALE_ALPHA,
+        BitDepth::Eight,
+    );
+}
+
 #[test]
 fn palette_8_should_be_rgb() {
     test_it_converts(
@@ -722,6 +746,18 @@ fn grayscale_alpha_8_should_be_grayscale_8() {
     );
 }
 
+#[test]
+fn grayscale_alpha_8_should_be_palette_8() {
+    test_it_converts(
+        "tests/files/grayscale_alpha_8_should_be_palette_8.png",
+        false,
+        GRAYSCALE_ALPHA,
+        BitDepth::Eight,
+        INDEXED,
+        BitDepth::Eight,
+    );
+}
+
 #[test]
 fn grayscale_16_should_be_grayscale_16() {
     test_it_converts(
@@ -926,6 +962,18 @@ fn grayscale_alpha_8_should_be_grayscale_trns_1() {
     );
 }
 
+#[test]
+fn grayscale_trns_8_should_be_grayscale_1() {
+    test_it_converts(
+        "tests/files/grayscale_trns_8_should_be_grayscale_1.png",
+        true,
+        GRAYSCALE,
+        BitDepth::Eight,
+        GRAYSCALE,
+        BitDepth::One,
+    );
+}
+
 #[test]
 fn small_files() {
     test_it_converts(
@@ -1012,6 +1060,45 @@ fn palette_should_be_reduced_with_unused() {
     remove_file(output).ok();
 }
 
+#[test]
+fn palette_should_be_reduced_with_bkgd() {
+    let input = PathBuf::from("tests/files/palette_should_be_reduced_with_bkgd.png");
+    let (output, opts) = get_opts(&input);
+
+    let png = PngData::new(&input, &opts).unwrap();
+
+    assert_eq!(png.raw.ihdr.color_type.png_header_code(), INDEXED);
+    assert_eq!(png.raw.ihdr.bit_depth, BitDepth::One);
+    assert_eq!(&png.aux_chunks[0].name, b"bKGD");
+    if let ColorType::Indexed { palette } = &png.raw.ihdr.color_type {
+        assert_eq!(palette.len(), 3);
+    }
+
+    match oxipng::optimize(&InFile::Path(input), &output, &opts) {
+        Ok(_) => (),
+        Err(x) => panic!("{}", x),
+    };
+    let output = output.path().unwrap();
+    assert!(output.exists());
+
+    let png = match PngData::new(output, &opts) {
+        Ok(x) => x,
+        Err(x) => {
+            remove_file(output).ok();
+            panic!("{}", x)
+        }
+    };
+
+    assert_eq!(png.raw.ihdr.color_type.png_header_code(), INDEXED);
+    assert_eq!(png.raw.ihdr.bit_depth, BitDepth::One);
+    assert_ne!(&png.aux_chunks[0].name, b"bKGD");
+    if let ColorType::Indexed { palette } = &png.raw.ihdr.color_type {
+        assert_eq!(palette.len(), 2);
+    }
+
+    remove_file(output).ok();
+}
+
 #[test]
 fn palette_should_be_reduced_with_both() {
     let input = PathBuf::from("tests/files/palette_should_be_reduced_with_both.png");
